@@ -67,6 +67,7 @@ def model(parm, priors, h, attrs, data, sd):
         if isinstance(prior, Uniform):
             index = attrs.index(prior.name)
             if parm[index] < prior.low or parm[index] > prior.high:
+     #           print "returning inf because " + prior.name + " is " + str(parm[index])
                 return -np.inf
 
         elif isinstance(prior, Normal):
@@ -78,25 +79,21 @@ def model(parm, priors, h, attrs, data, sd):
             ll += _lognormpdf(np.array(parm[indices]), np.array(prior.means),
                               prior.cov)
 
-    # print "parm after:", parm
     # Rebuild the hod dict from given vals
     hoddict = {attr:val for attr, val in zip(attrs, parm)}
 
-    # print h.__dict__
-
     h.update(**hoddict)
-    # print "updated"
-    # print h.__dict__
+
     # The logprob of the model
     model = h.corr_gal.copy()  # data + np.random.normal(scale=0.1)
-    # print "saved model"
+
     ll += np.sum(norm.logpdf(data, loc=model, scale=sd))
 
-    # print "got here"
+    # print ll
     return ll
 
 def fit_hod(r, data, sd, priors, guess=[], nwalkers=100, nsamples=100, burnin=10,
-           thin=50, nthreads=8, filename=None, **hodkwargs):
+           thin=50, nthreads=8, filename=None, chunks=None, **hodkwargs):
     """
     Run an MCMC procedure to fit a model correlation function to data
     
@@ -139,6 +136,13 @@ def fit_hod(r, data, sd, priors, guess=[], nwalkers=100, nsamples=100, burnin=10
     nthreads : int, default 1
         Number of threads to use in sampling.
         
+    filename : str, default ``None``
+        A path to a file to which to write results sequentially
+        
+    chunks : int, default ``None``
+        How many samples to run before appending results to file. Only
+        applicable if ``filename`` is provided.
+        
     \*\*hmfkwargs : arguments
         Any argument that could be sent to ``hmf.Perturbations``
         
@@ -201,14 +205,10 @@ def fit_hod(r, data, sd, priors, guess=[], nwalkers=100, nsamples=100, burnin=10
         guess = []
         for prior in priors:
             if isinstance(prior, Uniform):
-                print "uniform"
                 guess += [(prior.high + prior.low) / 2]
             elif isinstance(prior, Normal):
-                print "norm"
                 guess += [prior.mean]
             elif isinstance(prior, MultiNorm):
-                print "multinorm"
-                print prior.means
                 guess += prior.means.tolist()
 
     # Get an array of initial values
@@ -218,7 +218,19 @@ def fit_hod(r, data, sd, priors, guess=[], nwalkers=100, nsamples=100, burnin=10
     stacked_val = guess.copy()
     for i in range(nwalkers - 1):
         stacked_val = np.vstack((guess, stacked_val))
-    p0 = stacked_val * np.random.normal(loc=1.0, scale=0.2, size=ndim * nwalkers).reshape((nwalkers, ndim))
+
+    i = 0
+    for prior in priors:
+        if isinstance(prior, Uniform):
+            stacked_val[:, i] += np.random.normal(loc=0.0, scale=0.05 * (prior.high - prior.low), size=nwalkers)
+            i += 1
+        elif isinstance(prior, Normal):
+            stacked_val[:, i] += np.random.normal(loc=0.0, scale=prior.sd, size=nwalkers)
+            i += 1
+        elif isinstance(prior, MultiNorm):
+            for j, name in enumerate(prior.name):
+                stacked_val[:, i] += np.random.normal(loc=0.0, scale=np.sqrt(prior.cov[j, j]), size=nwalkers)
+                i += 1
 
     sampler = emcee.EnsembleSampler(nwalkers, ndim, model,
                                     args=[priors, h, attrs, data, sd],
@@ -226,12 +238,30 @@ def fit_hod(r, data, sd, priors, guess=[], nwalkers=100, nsamples=100, burnin=10
 
     # Run a burn-in
     if burnin:
-        pos, prob, state = sampler.run_mcmc(p0, burnin)
+        pos, prob, state = sampler.run_mcmc(stacked_val, burnin)
         sampler.reset()
     else:
-        pos = p0
+        pos = stacked_val
+
+
     # Run the actual run
-    sampler.run_mcmc(pos, nsamples, thin=thin)
+    if filename is None:
+        sampler.run_mcmc(pos, nsamples, thin=thin)
+    else:
+        header = "# " + "\t".join(attrs) + "\n"
+        with open(filename, "w") as f:
+            f.write(header)
+
+        if chunks is None:
+            chunks = 1
+        else:
+            if nsamples % chunks != 0:
+                print "Warning: nsamples must be a multiple of chunks, setting to 1"
+                chunks = 1
+        for i, result in enumerate(sampler.sample(pos, iterations=nsamples)):
+            if i % chunks == 0:
+                with open(filename, 'a') as f:
+                    np.savetxt(f, sampler.flatchain[-chunks:, :])
 
     return sampler.flatchain, np.mean(sampler.acceptance_fraction)
 
