@@ -63,7 +63,7 @@ class HOD(object):
     
     '''
     def __init__(self, M=np.linspace(8, 18, 500), r=np.linspace(1, 200, 200),
-                  M_1=12.851, alpha=1.049, M_min=11.6222,
+                 M_1=12.851, alpha=1.049, M_min=11.6222,
                  gauss_width=0.26, M_0=11.5047, fca=0.5, fcb=0, fs=1,
                  delta=None, x=1, hod_model='zehavi',
                  halo_profile='nfw', cm_relation='duffy', bias_model='tinker',
@@ -487,6 +487,7 @@ class HOD(object):
         """The 2-halo term of the galaxy power spectrum - NOT LOGGED"""
         if not self.scale_dependent_bias and self.halo_exclusion in [None, "schneider"]:
             u = self.profile.u(np.exp(self.pert.lnk), self.pert.M, self.pert.z)
+
             pg2h = fort.power_gal_2h(nlnk=len(self.pert.lnk),
                                      nm=len(self.pert.M),
                                      u=np.asfortranarray(u),
@@ -568,13 +569,11 @@ class HOD(object):
                 if self.halo_exclusion != "ellipsoid":
                     pg2h[i] = intg.simps(integrand, m)
 
-        self.pre_pg2h = pg2h
         pg2h = np.exp(self.matter_power) * pg2h ** 2 / self.mean_gal_den ** 2
 
         if self.halo_exclusion == 'schneider':
             # should be k,r/h, but we use k/h and r (which gives the same)
-            pg2h *= tools.exclusion_window(np.exp(self.pert.lnk), r=2)
-        self.post_pg2h = pg2h
+            pg2h *= np.abs(tools.exclusion_window(np.exp(self.pert.lnk), r=2))
         return pg2h
 
     @property
